// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Map;
import java.util.HashMap;
import javax.swing.*;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;
import com.intellij.openapi.options.ConfigurationException;

import com.kiwisoft.generate.model.Template;
import com.kiwisoft.generate.model.TemplateManager;
import com.kiwisoft.generate.model.ui.TemplateDialog;
import com.kiwisoft.utils.ListenerSupport;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.gui.table.SortableTable;
import com.kiwisoft.utils.gui.table.SortableTableRow;
import com.kiwisoft.utils.gui.table.DefaultSortableTableModel;
import com.kiwisoft.utils.gui.table.PropertiesTableModel;
import com.kiwisoft.utils.gui.Icons;
import com.kiwisoft.utils.idea.PluginUtils;

/**
 * Main template configuration panel.
 *
 * @author Stefan Stiller
 */
public class ConfigurationPanel extends JPanel
{
	private ListenerSupport listenerSupport=new ListenerSupport();

	private TemplatesTableModel tmTemplates;
	private SortableTable tblTemplates;
	private PropertiesTableModel tmParameters;
	private SortableTable tblParameters;

	private boolean modified;

	public ConfigurationPanel()
	{
		initializeComponents();
	}

	/**
	 * Initialize all UI components.
	 */
	private void initializeComponents()
	{
		int row=0;
		setLayout(new GridBagLayout());
		add(createTemplatesPanel(),
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(4, 4, 4, 4), 0, 0));
		add(createParametersPanel(),
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(4, 4, 4, 4), 0, 0));
		add(Box.createGlue(),
			new GridBagConstraints(0, row, 1, 1, 0.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.HORIZONTAL, new Insets(0, 0, 0, 0), 0, 0));
	}

	private JPanel createTemplatesPanel()
	{
		tmTemplates=new TemplatesTableModel();
		tblTemplates=new SortableTable(tmTemplates);
		tblTemplates.putClientProperty(SortableTable.AUTO_RESIZE_ROWS, Boolean.TRUE);
		DefaultTableConfiguration tableConfig=new DefaultTableConfiguration("templates");
		tableConfig.setSupportWidths(false);
		tblTemplates.initializeColumns(tableConfig);
		tmTemplates.addSortColumn(0, false);
		tblTemplates.setShowGrid(false);
		if (tblTemplates.getRowCount()>0) tblTemplates.getSelectionModel().addSelectionInterval(0, 0);
		tblTemplates.addMouseListener(new MouseAdapter()
		{
			public void mouseClicked(MouseEvent e)
			{
				if (e.getButton()==MouseEvent.BUTTON1 && e.getClickCount()>1)
				{
					int row=tblTemplates.rowAtPoint(e.getPoint());
					if (row>=0) openEditor(tmTemplates.getRow(row));
					e.consume();
				}
			}
		});

		JScrollPane scrlTemplates=new JScrollPane(tblTemplates);
		scrlTemplates.setPreferredSize(new Dimension(400, 150));
		scrlTemplates.getViewport().setBackground(tblTemplates.getBackground());

		JPanel panel=new JPanel(new GridBagLayout());
		panel.setBorder(new TitledBorder(new EtchedBorder(), "Templates"));
		panel.add(createTemplatesToolbar(),
			new GridBagConstraints(0, 0, 1, 1, 0.0, 1.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
		panel.add(scrlTemplates,
			new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(4, 4, 4, 4), 0, 0));
		return panel;
	}

	private JComponent createTemplatesToolbar()
	{
		DefaultActionGroup actionGroup=new DefaultActionGroup("KiwiCreate.Configuration.Templates", false);
		actionGroup.add(new AddTemplateAction());
		actionGroup.add(new RemoveTemplateAction());
		actionGroup.addSeparator();
		actionGroup.add(new EditTemplateAction());
		actionGroup.add(new CopyTemplateAction());
		ActionManager actionManager=ActionManager.getInstance();
		return actionManager.createActionToolbar("KiwiCreate.Configuration.Templates", actionGroup, true).getComponent();
	}

	private JPanel createParametersPanel()
	{
		tmParameters=new PropertiesTableModel(TemplateManager.getInstance().getParameters());
		tblParameters=new SortableTable(tmParameters);
		tblParameters.putClientProperty(SortableTable.AUTO_RESIZE_ROWS, Boolean.TRUE);
		DefaultTableConfiguration tableConfig=new DefaultTableConfiguration("parameters");
		tableConfig.setSupportWidths(false);
		tblParameters.initializeColumns(tableConfig);
		tmParameters.addSortColumn(0, false);
		if (tblParameters.getRowCount()>0) tblParameters.getSelectionModel().addSelectionInterval(0, 0);

		JScrollPane scrollPane=new JScrollPane(tblParameters);
		scrollPane.setPreferredSize(new Dimension(300, 150));
		scrollPane.getViewport().setBackground(tblParameters.getBackground());

		JPanel panel=new JPanel(new GridBagLayout());
		panel.setBorder(new TitledBorder(new EtchedBorder(), "Static Parameters"));
		panel.add(createParametersToolbar(),
			new GridBagConstraints(0, 0, 1, 1, 0.0, 1.0, GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(4, 4, 4, 4), 0, 0));
		panel.add(scrollPane,
			new GridBagConstraints(0, 1, 1, 1, 1.0, 1.0, GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(4, 4, 4, 4), 0, 0));
		return panel;
	}

	private JComponent createParametersToolbar()
	{
		DefaultActionGroup actionGroup=new DefaultActionGroup("KiwiCreate.Configuration.Parameters", false);
		actionGroup.add(new AddParameterAction());
		actionGroup.add(new RemoveParameterAction());
		ActionManager actionManager=ActionManager.getInstance();
		return actionManager.createActionToolbar("KiwiCreate.Configuration.Parameters", actionGroup, true).getComponent();
	}

	/**
	 * Uninstall all listeners
	 */
	private void removeListeners()
	{
		listenerSupport.dispose();
	}

	/**
	 * @return <code>true</code> if the user modified the templates.
	 */
	public boolean isModified()
	{
		return modified || (tmParameters!=null && tmParameters.isModified());
	}

	/**
	 * Adds a template to the table.
	 *
	 * @param template The template to be added
	 */
	private void addTemplate(Template template)
	{
		tmTemplates.addTemplate(template);
		int index=tmTemplates.indexOf(template);
		if (index>=0) tblTemplates.getSelectionModel().setSelectionInterval(index, index);
	}

	/**
	 * Saves the modified templates and updates the context actions.
	 */
	public void apply() throws ConfigurationException
	{
		Map<String, String> parameters=new HashMap<String, String>();
		for (int i=0;i<tmParameters.getRowCount();i++)
		{
			PropertiesTableModel.Row row=(PropertiesTableModel.Row)tmParameters.getRow(i);
			if (StringUtils.isEmpty(row.getUserObject()))
			{
				throw new ConfigurationException("Missing name for parameter.");
			}
			if (parameters.containsKey(row.getUserObject()))
			{
				throw new ConfigurationException("Duplicate parameter '"+row.getUserObject()+"'.");
			}
			parameters.put(row.getUserObject(), row.getValue());
		}
		TemplateManager manager=TemplateManager.getInstance();
		manager.setParameters(parameters);
		manager.removeAllTemplates();
		for (int i=0; i<tmTemplates.getRowCount(); i++)
		{
			Template template=tmTemplates.getObject(i);
			manager.addTemplate(template);
		}
		manager.save();
		GeneratorPlugin.initializeActions();
		modified=false;
	}

	/**
	 * Reset all modifications.
	 */
	public void reset()
	{
		tmTemplates.clear();
		tmTemplates.createData();
		modified=false;
	}

	/**
	 * Dispose all resources.
	 */
	public void dispose()
	{
		removeListeners();
	}

	private void openEditor(SortableTableRow<Template> row)
	{
		Template template=row.getUserObject().copy();
		TemplateDialog dialog=new TemplateDialog(ConfigurationPanel.this, template);
		PluginUtils.showDialog(dialog, true, true);
		if (dialog.isOK())
		{
			row.setUserObject(template);
			modified=true;
		}
	}

	private class EditTemplateAction extends AnAction
	{
		private EditTemplateAction()
		{
			super("Edit Template", "Edit the selected template.", Icons.getIcon("edit"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			e.getPresentation().setEnabled(tblTemplates.getSelectedRowCount()==1);
		}

		public void actionPerformed(AnActionEvent e)
		{
			int rowIndex=tblTemplates.getSelectedRow();
			if (rowIndex>=0)
			{
				SortableTableRow<Template> row=tmTemplates.getRow(rowIndex);
				if (row!=null) openEditor(row);
			}
		}
	}

	private class CopyTemplateAction extends AnAction
	{
		private CopyTemplateAction()
		{
			super("Copy Template", "Copy the selected template.", Icons.getIcon("copy"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			e.getPresentation().setEnabled(tblTemplates.getSelectedRowCount()==1);
		}

		public void actionPerformed(AnActionEvent e)
		{
			int rowIndex=tblTemplates.getSelectedRow();
			if (rowIndex>=0)
			{
				SortableTableRow<Template> row=tmTemplates.getRow(rowIndex);
				if (row!=null)
				{
					final Template template=row.getUserObject().copy();
					template.setName(template.getName()+" (2)");
					TemplateDialog dialog=new TemplateDialog(ConfigurationPanel.this, template);
					PluginUtils.showDialog(dialog, true, true);
					if (dialog.isOK())
					{
						addTemplate(template);
						modified=true;
					}
				}
			}
		}
	}

	private class RemoveTemplateAction extends AnAction
	{
		private RemoveTemplateAction()
		{
			super("Delete Template", "Deletes the selected templates.", Icons.getIcon("remove"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			e.getPresentation().setEnabled(tblTemplates.getSelectedRow()>=0);
		}

		public void actionPerformed(AnActionEvent e)
		{
			modified=tmTemplates.removeSelectedRows(tblTemplates) || modified;
		}
	}

	private class AddTemplateAction extends AnAction
	{
		private AddTemplateAction()
		{
			super("Create Template", "Creates a new template and opens it for editing.", Icons.getIcon("add"));
		}

		public void actionPerformed(AnActionEvent e)
		{
			final Template template=new Template();
			TemplateDialog dialog=new TemplateDialog(ConfigurationPanel.this, template);
			PluginUtils.showDialog(dialog, true, true);
			if (dialog.isOK())
			{
				addTemplate(template);
				modified=true;
			}
		}
	}

	private class AddParameterAction extends AnAction
	{
		private AddParameterAction()
		{
			super("Create Parameter", "Creates a new parameter.", Icons.getIcon("add"));
		}

		public void actionPerformed(AnActionEvent e)
		{
			tmParameters.createRow();
			modified=true;
		}
	}

	private class RemoveParameterAction extends AnAction
	{
		private RemoveParameterAction()
		{
			super("Remove Parameter", "Deletes all selected parameters.", Icons.getIcon("remove"));
		}

		public void update(AnActionEvent e)
		{
			super.update(e);
			e.getPresentation().setEnabled(tblParameters.getSelectedRow()>=0);
		}

		public void actionPerformed(AnActionEvent e)
		{
			modified=tmParameters.removeSelectedRows(tblParameters) || modified;
		}
	}

	private class TemplatesTableModel extends DefaultSortableTableModel<Template>
	{
		private TemplatesTableModel()
		{
			super("name");
			createData();
		}

		private void createData()
		{
			TemplateManager templateManager=TemplateManager.getInstance();
			for (Template template : templateManager.getTemplates())
			{
				addRow(new Row(template.copy()));
			}
			sort();
		}

		private void addTemplate(Template template)
		{
			addRow(new Row(template));
			sort();
		}

		private class Row extends SortableTableRow<Template>
		{
			public Row(Template template)
			{
				super(template);
			}

			public Comparable getSortValue(int col)
			{
				return getUserObject().getName();
			}

			public Object getDisplayValue(int col)
			{
				return getUserObject();
			}
		}
	}
}
