// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import static java.awt.GridBagConstraints.BOTH;
import static java.awt.GridBagConstraints.CENTER;
import java.awt.GridBagConstraints;
import java.awt.Insets;

import org.apache.commons.beanutils.DynaBean;

import com.kiwisoft.generate.model.UserParameter;
import com.kiwisoft.generate.model.UserType;
import com.kiwisoft.generate.model.ParameterBean;

/**
 * A panel for an optional user type property.
 *
 * @author Stefan Stiller
 */
class OptionalGroupPanel extends TitledParameterPanel
{
	private ParameterBean bean;
	private GroupPanel groupPanel;

	public OptionalGroupPanel(ParameterBean bean, UserParameter parameter)
	{
		super(parameter);
		this.bean=bean;
		initializePanel();
	}

	private void initializePanel()
	{
		ParameterBean value=(ParameterBean)bean.get(getParameter().getName());
		if (value!=null)
		{
			groupPanel=new GroupPanel(value, (UserType)getParameter().getType(), new RemoveElementAction(value));
			getElementsPanel().add(groupPanel, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0, CENTER, BOTH, new Insets(2, 2, 2, 0), 0, 0));
		}
	}

	protected boolean canAddElement()
	{
		return bean!=null && bean.get(getParameter().getName())==null;
	}

	protected void addElement()
	{
		if (bean.get(getParameter().getName())==null)
		{
			UserType userType=(UserType)getParameter().getType();
			try
			{
				DynaBean newBean=userType.createDynaClass().newInstance();
				bean.set(getParameter().getName(), newBean);
				initializePanel();
				getElementsPanel().updateUI();
			}
			catch (Exception e)
			{
				throw new RuntimeException(e);
			}
		}
	}

	protected boolean canRemoveElement(Object value)
	{
		return !getParameter().isRequired() && bean!=null && groupPanel!=null;
	}

	protected void removeElement(Object value)
	{
		if (groupPanel!=null)
		{
			bean.set(getParameter().getName(), null);
			groupPanel.dispose();
			getElementsPanel().remove(groupPanel);
			getElementsPanel().updateUI();
		}
	}
}
