// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate;

import java.awt.BorderLayout;
import java.awt.GridBagLayout;
import javax.swing.JPanel;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.BorderFactory;
import javax.swing.border.BevelBorder;

import com.intellij.openapi.actionSystem.DefaultActionGroup;
import com.intellij.openapi.actionSystem.ActionManager;
import com.intellij.openapi.actionSystem.AnAction;
import com.intellij.openapi.actionSystem.AnActionEvent;

import com.kiwisoft.utils.Disposable;
import com.kiwisoft.utils.Utils;
import com.kiwisoft.utils.gui.Icons;
import com.kiwisoft.generate.model.UserParameter;

/**
 * A panel with a title row in a darker color.
 *
 * @author Stefan Stiller
 */
class TitledParameterPanel extends JPanel implements Disposable
{
	private UserParameter parameter;
	private JPanel elementsPanel;

	public TitledParameterPanel(UserParameter parameter)
	{
		this.parameter=parameter;
		DefaultActionGroup actionGroup=new DefaultActionGroup("KiwiCreate.ParameterDialog", false);
		actionGroup.add(new AddElementAction());
		ActionManager actionManager=ActionManager.getInstance();
		JComponent toolbar=actionManager.createActionToolbar("KiwiCreate.ParameterDialog", actionGroup, true).getComponent();
		toolbar.setOpaque(false);

		JLabel label=new JLabel(parameter.getDisplayableName()+":");
		label.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		label.setToolTipText(parameter.getDescription());

		JPanel titlePanel=new JPanel(new BorderLayout());
		titlePanel.setBackground(Utils.darker(titlePanel.getBackground(), 0.9));
		titlePanel.add(label, BorderLayout.CENTER);
		titlePanel.add(toolbar, BorderLayout.EAST);

		elementsPanel=new JPanel(new GridBagLayout());

		setLayout(new BorderLayout());
		setBorder(BorderFactory.createBevelBorder(BevelBorder.RAISED));
		add(titlePanel, BorderLayout.NORTH);
		add(elementsPanel, BorderLayout.CENTER);
	}

	public UserParameter getParameter()
	{
		return parameter;
	}

	public JPanel getElementsPanel()
	{
		return elementsPanel;
	}

	protected boolean canAddElement()
	{
		return false;
	}

	protected void addElement()
	{
	}

	protected boolean canRemoveElement(Object value)
	{
		return false;
	}

	protected void removeElement(Object value)
	{
	}

	public void dispose()
	{
	}

	private class AddElementAction extends AnAction
	{
		public AddElementAction()
		{
			super("Add", "Add a new instance", Icons.getIcon("add"));
		}

		public void update(AnActionEvent event)
		{
			super.update(event);
			event.getPresentation().setEnabled(canAddElement());
		}

		public void actionPerformed(AnActionEvent event)
		{
			addElement();
		}
	}

	protected class RemoveElementAction extends AnAction
	{
		private Object value;

		public RemoveElementAction(Object value)
		{
			super("Remove", "Remove this element", Icons.getIcon("remove"));
			this.value=value;
		}

		public void update(AnActionEvent event)
		{
			super.update(event);
			event.getPresentation().setEnabled(canRemoveElement(value));
		}

		public void actionPerformed(AnActionEvent event)
		{
			removeElement(value);
		}
	}
}
