// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model;

import java.util.List;

import org.apache.commons.beanutils.DynaClass;

import com.kiwisoft.utils.ObservableDynaBean;

/**
 * A bean for storing the parameter values of the template model.
 *
 * @author Stefan Stiller
 */
public class ParameterBean extends ObservableDynaBean
{
	public ParameterBean(DynaClass dynaClass)
	{
		super(dynaClass);
	}

	public ParameterClass getDynaClass()
	{
		return (ParameterClass)super.getDynaClass();
	}

	public boolean isValid()
	{
		UserType userType=getDynaClass().getParameterGroup();
		for (UserParameter parameter : userType.getParameters())
		{
			if (parameter.getType() instanceof UserType)
			{
				if (parameter.isCollection())
				{
					List<ParameterBean> elements=(List<ParameterBean>)get(parameter.getName());
					if (parameter.isRequired())
					{
						if (elements==null || elements.isEmpty()) return false;
					}
					if (elements!=null)
					{
						for (ParameterBean bean : elements)
						{
							if (!bean.isValid()) return false;
						}
					}
				}
				else
				{
					ParameterBean value=(ParameterBean)get(parameter.getName());
					if (parameter.isRequired() && value==null) return false;
					if (value!=null && !value.isValid()) return false;
				}
			}
			else
			{
				if (parameter.isCollection())
				{
					List elements=(List)get(parameter.getName());
					if (parameter.isRequired())
					{
						if (elements==null || elements.isEmpty()) return false;
					}
				}
				else
				{
					Object value=get(parameter.getName());
					if (parameter.isRequired())
					{
						if (value==null || "".equals(value)) return false;
					}
				}
			}
		}
		return true;
	}
}
