// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model;

import java.util.ArrayList;
import java.util.List;

import org.apache.commons.beanutils.BasicDynaClass;
import org.apache.commons.beanutils.DynaBean;
import org.apache.commons.beanutils.DynaProperty;
import org.apache.commons.beanutils.DynaClass;

/**
 * The {@link DynaClass} of a {@link ParameterBean}.
 *
 * @author Stefan Stiller
 */
public class ParameterClass extends BasicDynaClass
{
	private UserType userType;

	public ParameterClass(UserType userType)
	{
		super(userType.getName(), ParameterBean.class);
		this.userType=userType;
		List<DynaProperty> dynaProperties=new ArrayList<DynaProperty>();
		for (UserParameter parameter : userType.getParameters())
		{
			if (parameter.isCollection())
				dynaProperties.add(new DynaProperty(parameter.getName(), List.class));
			else
				dynaProperties.add(new DynaProperty(parameter.getName(), parameter.getType().getTypeClass()));
		}
		setProperties(dynaProperties.toArray(new DynaProperty[0]));
	}

	public DynaBean newInstance() throws IllegalAccessException, InstantiationException
	{
		DynaBean bean=super.newInstance();
		for (UserParameter parameter : userType.getParameters())
		{
			if (parameter.isCollection())
			{
				List<Object> list=new ArrayList<Object>();
				if (parameter.getType() instanceof UserType && parameter.isRequired())
				{
					DynaClass dynaClass=((UserType)parameter.getType()).createDynaClass();
					list.add(dynaClass.newInstance());
				}
				bean.set(parameter.getName(), list);
			}
			else
			{
				if (parameter.getType() instanceof UserType)
				{
					if (parameter.isRequired())
					{
						DynaClass dynaClass=((UserType)parameter.getType()).createDynaClass();
						bean.set(parameter.getName(), dynaClass.newInstance());
					}
				}
				else
				{
					bean.set(parameter.getName(), parameter.getDefaultValue());
				}
			}
		}
		return bean;
	}

	public UserType getParameterGroup()
	{
		return userType;
	}
}
