// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;

import com.kiwisoft.utils.ObservableBean;
import com.kiwisoft.utils.StringUtils;
import com.kiwisoft.utils.xml.XMLAdapter;
import com.kiwisoft.utils.xml.XMLContext;
import com.kiwisoft.utils.xml.XMLWriter;

/**
 * A template file. 
 *
 * @author Stefan Stiller
 */
public class TemplateFile extends XMLAdapter implements ObservableBean
{
	private String name;
	private String fileName;
	private String content;
	private boolean overwrite;
	private boolean openEditor;

	private PropertyChangeSupport changeSupport=new PropertyChangeSupport(this);

	public TemplateFile()
	{
	}

	public String getName()
	{
		return name;
	}

	public void setName(String name)
	{
		String oldValue=this.name;
		this.name=name;
		firePropertyChange("name", oldValue, name);
	}

	public String getFileName()
	{
		return fileName;
	}

	public void setFileName(String fileName)
	{
		String oldValue=this.fileName;
		this.fileName=fileName;
		firePropertyChange("fileName", oldValue, fileName);
	}

	public String getContent()
	{
		return content;
	}

	public void setContent(String content)
	{
		String oldValue=this.content;
		this.content=content;
		firePropertyChange("content", oldValue, content);
	}

	public boolean isOverwrite()
	{
		return overwrite;
	}

	public void setOverwrite(boolean overwrite)
	{
		boolean oldValue=this.overwrite;
		this.overwrite=overwrite;
		firePropertyChange("overwrite", oldValue, overwrite);
	}

	public boolean isOpenEditor()
	{
		return openEditor;
	}

	public void setOpenEditor(boolean openEditor)
	{
		boolean oldValue=this.openEditor;
		this.openEditor=openEditor;
		firePropertyChange("openEditor", oldValue, openEditor);
	}

	// PropertyChangeSupport delegates

	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(propertyName, listener);
	}

	public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(propertyName, listener);
	}

	public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
	{
		changeSupport.firePropertyChange(propertyName, oldValue, newValue);
	}

	// XMLObject interface methods

	public TemplateFile(XMLContext context, String name)
	{
		super(context, name);
	}

	public void setXMLAttribute(XMLContext context, String name, String value)
	{
		if ("name".equalsIgnoreCase(name))
			this.name=value;
		else if ("fileName".equalsIgnoreCase(name))
			this.fileName=value;
		else if ("overwrite".equalsIgnoreCase(name))
			this.overwrite=Boolean.parseBoolean(value);
		else if ("openEditor".equalsIgnoreCase(name))
			this.openEditor=Boolean.parseBoolean(value);
	}

	public void writeXML(XMLWriter xmlWriter) throws IOException
	{
		xmlWriter.startElement("file");
		xmlWriter.setAttribute("name", name);
		xmlWriter.setAttribute("fileName", fileName);
		xmlWriter.setAttribute("overwrite", overwrite);
		xmlWriter.setAttribute("openEditor", openEditor);
		xmlWriter.closeElement("file");
	}

	public TemplateFile copy()
	{
		TemplateFile file=new TemplateFile();
		file.setName(getName());
		file.setFileName(getFileName());
		file.setOverwrite(isOverwrite());
		file.setOpenEditor(isOpenEditor());
		file.setContent(getContent());
		return file;
	}

	public void validate()
	{
		if (StringUtils.isEmpty(getName()))
			throw new RuntimeException("Missing template file name.");
		if (StringUtils.isEmpty(getFileName()))
			throw new RuntimeException("Missing file name pattern for template file '"+getName()+"'.");
	}
}
