// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.model.ui;

import java.awt.Color;
import java.awt.GridBagConstraints;
import static java.awt.GridBagConstraints.*;
import java.awt.Insets;
import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;

import com.kiwisoft.generate.DefaultTableConfiguration;
import com.kiwisoft.generate.model.SystemMethod;
import com.kiwisoft.generate.model.SystemParameter;
import com.kiwisoft.utils.gui.table.DefaultSortableTableModel;
import com.kiwisoft.utils.gui.table.SortableTable;
import com.kiwisoft.utils.gui.table.SortableTableRow;

/**
 * A panel containing details for a hard coded method.  
 *
 * @author Stefan Stiller
 */
public class SystemMethodPanel extends DetailsPanel<SystemMethod>
{
	public SystemMethodPanel(SystemMethod method)
	{
		super(method);
		initComponents();
	}

	private void initComponents()
	{
		JTextField nameField=new JTextField();
		nameField.setEditable(false);
		JTextArea descriptionField=new JTextArea();
		descriptionField.setEditable(false);
		descriptionField.setWrapStyleWord(true);
		descriptionField.setLineWrap(true);
		descriptionField.setRows(3);
		JScrollPane descriptionPanel=new JScrollPane(descriptionField);

		int row=0;
		add(new JLabel("Name:"),
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, WEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		add(nameField,
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, WEST, HORIZONTAL, new Insets(0, 5, 5, 5), 0, 0));
		add(new JLabel("Description:"),
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, WEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		add(descriptionPanel,
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.3, WEST, BOTH, new Insets(0, 5, 5, 5), 0, 0));
		add(new JLabel("Parameters:"),
			new GridBagConstraints(0, row++, 1, 1, 1.0, 0.0, WEST, NONE, new Insets(5, 5, 5, 5), 0, 0));
		add(new JScrollPane(createParameterTable()),
			new GridBagConstraints(0, row, 1, 1, 1.0, 0.7, WEST, BOTH, new Insets(0, 5, 5, 5), 0, 0));

		nameField.setText(getObject().getName());
		descriptionField.setText(getObject().getDescription());
	}

	protected SortableTable createParameterTable()
	{
		DefaultSortableTableModel<SystemParameter> model=new DefaultSortableTableModel<SystemParameter>("name", "type", "description");
		model.setResortable(false);
		for (SystemParameter parameter : getObject().getParameter()) model.addRow(new ParameterRow(parameter));
		SortableTable table=new SortableTable(model);
		table.putClientProperty(SortableTable.AUTO_RESIZE_ROWS, true);
		table.putClientProperty(SortableTable.ALTERNATE_ROW_BACKGROUND, new Color(220, 220, 255));
		DefaultTableConfiguration configuration=new DefaultTableConfiguration("method.parameters");
		configuration.setSupportWidths(false);
		table.initializeColumns(configuration);
		table.sizeColumnsToFit(true, true);
		table.sizeRowsToFit();
		return table;
	}

	private static class ParameterRow extends SortableTableRow<SystemParameter>
	{
		public ParameterRow(SystemParameter parameter)
		{
			super(parameter);
		}

		public String getCellFormat(int col)
		{
			if (col==2) return "MultiLine";
			return super.getCellFormat(col);
		}

		public Object getDisplayValue(int column)
		{
			switch (column)
			{
				case 0:
					return getUserObject().getName();
				case 1:
					return getUserObject().getType();
				case 2:
					return getUserObject().getDescription();
			}
			return null;
		}
	}
}
