// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.tree;

import java.util.Vector;
import java.awt.Font;

import javax.swing.JTree;

import com.kiwisoft.utils.gui.tree.GenericTreeNode;
import com.kiwisoft.utils.CollectionChangeListener;
import com.kiwisoft.utils.CollectionChangeEvent;
import com.kiwisoft.generate.model.Template;
import com.kiwisoft.generate.model.TemplateFile;

/**
 * A tree node used as container for all template files. 
 *
 * @author Stefan Stiller
 */
public class FilesNode extends GenericTreeNode<String> implements CollectionChangeListener
{
	private Template template;

	public FilesNode(Template template)
	{
		super("Files");
		this.template=template;
		template.addListener(this);
	}

	public Vector<GenericTreeNode> loadChildren()
	{
		Vector<GenericTreeNode> children=super.loadChildren();
		for (TemplateFile file : template.getFiles())
		{
			children.add(new TemplateFileNode(file));
		}
		return children;
	}

	public void collectionChanged(CollectionChangeEvent event)
	{
		if (event.getType()==CollectionChangeEvent.ADDED)
		{
			if ("files".equals(event.getPropertyName()))
			{
				addChild(new TemplateFileNode((TemplateFile)event.getElement()));
			}
		}
		else if (event.getType()==CollectionChangeEvent.REMOVED)
		{
			if ("files".equals(event.getPropertyName()))
			{
				removeObject(event.getElement());
			}
		}
	}

	public void dispose()
	{
		template.removeListener(this);
		super.dispose();
	}

	public Font getFont(JTree tree)
	{
		return tree.getFont().deriveFont(Font.BOLD);
	}
}
