// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.tree;

import java.awt.Font;
import java.util.Vector;
import javax.swing.JTree;

import com.kiwisoft.generate.model.SystemParameter;
import com.kiwisoft.generate.model.Types;
import com.kiwisoft.generate.model.Type;
import com.kiwisoft.utils.gui.tree.GenericTreeNode;
import com.kiwisoft.utils.gui.tree.TreeNodeLeaf;

/**
 * A tree node used as parent for all hardcoded parameters.
 *
 * @author Stefan Stiller
 */
public class SystemParametersNode extends GenericTreeNode<String>
{
	public SystemParametersNode()
	{
		super("Predefined Parameters");
	}

	public Vector<GenericTreeNode> loadChildren()
	{
		Vector<GenericTreeNode> nodes=super.loadChildren();
		Type stringType=Types.valueOf("String");
		Type intType=Types.valueOf("Integer");
		nodes.add(new GenericTreeNode<String>(
			"Dates",
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("day", "Current day (1..31)", intType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("month", "Current month (January=1, February=2,...)", intType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("today", "Current date in the current locale default format", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("year", "Current year (2006)", intType))));
		nodes.add(new GenericTreeNode<String>(
			"Selection",
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("directory", "The directory of the selected item.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("file", "The file name of the selected item.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("module", "The module name of the selected item.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("moduleDirectory", "The path containing the module file of the selected item.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("package", "The package of the selected item.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("project", "The name of the project.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("projectDirectory", "The path containing the project file.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("rootDirectory", "The root source path containing the selected item.", stringType)),
			new TreeNodeLeaf<SystemParameter>(new SystemParameter("relativeSourcePath", "The relative path of the source root directory to the directory of the selected item.",
								stringType))));
		return nodes;
	}

	public Font getFont(JTree tree)
	{
		return tree.getFont().deriveFont(Font.BOLD);
	}
}
