// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.tree;

import java.util.Vector;
import java.awt.Font;

import javax.swing.JTree;

import com.kiwisoft.generate.model.Template;
import com.kiwisoft.generate.model.UserParameter;
import com.kiwisoft.utils.gui.tree.GenericTreeNode;
import com.kiwisoft.utils.CollectionChangeListener;
import com.kiwisoft.utils.CollectionChangeEvent;

/**
 * A tree node used as parent for all user defined model parameters. 
 *
 * @author Stefan Stiller
 */
public class UserParametersNode extends GenericTreeNode<String> implements CollectionChangeListener
{
	private Template template;

	public UserParametersNode(Template template)
	{
		super("User Parameters");
		this.template=template;
		template.addListener(this);
	}

	public Vector<GenericTreeNode> loadChildren()
	{
		Vector<GenericTreeNode> children=super.loadChildren();
		for (UserParameter parameter : template.getParameters())
		{
			children.add(new ParameterNode(parameter));
		}
		return children;
	}

	public void collectionChanged(CollectionChangeEvent event)
	{
		if (event.getType()==CollectionChangeEvent.ADDED)
		{
			if ("parameters".equals(event.getPropertyName()))
			{
				addChild(new ParameterNode((UserParameter)event.getElement()));
			}
		}
		else if (event.getType()==CollectionChangeEvent.REMOVED)
		{
			if ("parameters".equals(event.getPropertyName()))
			{
				removeObject(event.getElement());
			}
		}
	}

	public void dispose()
	{
		template.removeListener(this);
		super.dispose();
	}

	public boolean moveUp(GenericTreeNode childNode)
	{
		if (super.moveUp(childNode))
		{
			if (childNode instanceof ParameterNode)
			{
				//noinspection unchecked
				template.moveUp(((ParameterNode)childNode).getUserObject());
			}
			return true;
		}
		return false;
	}

	public boolean moveDown(GenericTreeNode childNode)
	{
		if (super.moveDown(childNode))
		{
			if (childNode instanceof ParameterNode)
			{
				//noinspection unchecked
				template.moveDown(((ParameterNode)childNode).getUserObject());
			}
			return true;
		}
		return false;
	}

	public Font getFont(JTree tree)
	{
		return tree.getFont().deriveFont(Font.BOLD);
	}
}
