// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.generate.tree;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.util.Vector;

import com.kiwisoft.generate.model.UserParameter;
import com.kiwisoft.generate.model.UserType;
import com.kiwisoft.utils.gui.tree.GenericTreeNode;
import com.kiwisoft.utils.gui.ObjectStyle;
import com.kiwisoft.utils.CollectionChangeListener;
import com.kiwisoft.utils.CollectionChangeEvent;

/**
 * A tree node representing a user defined type. 
 *
 * @author Stefan Stiller
 */
public class UserTypeNode extends GenericTreeNode<UserType> implements PropertyChangeListener, CollectionChangeListener
{
	public UserTypeNode(UserType type)
	{
		super(type);
		type.addPropertyChangeListener(this);
		type.addListener(this);
	}

	public Vector<GenericTreeNode> loadChildren()
	{
		Vector<GenericTreeNode> children=super.loadChildren();
		for (UserParameter parameter : getUserObject().getParameters())
		{
			children.add(new ParameterNode(parameter));
		}
		return children;
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		if ("name".equals(evt.getPropertyName())) fireNodeChanged();
	}

	public ObjectStyle getStyle()
	{
		if (!getUserObject().isValid()) return INVALID_STYLE;
		return super.getStyle();
	}

	public void collectionChanged(CollectionChangeEvent event)
	{
		if (event.getType()==CollectionChangeEvent.ADDED)
		{
			if ("parameters".equals(event.getPropertyName()))
			{
				addChild(new ParameterNode((UserParameter)event.getElement()));
			}
		}
		else if (event.getType()==CollectionChangeEvent.REMOVED)
		{
			if ("parameters".equals(event.getPropertyName()))
			{
				removeObject(event.getElement());
			}
		}
	}

	public void dispose()
	{
		getUserObject().removePropertyChangeListener(this);
		getUserObject().removeListener(this);
		super.dispose();
	}

	public boolean moveUp(GenericTreeNode childNode)
	{
		if (super.moveUp(childNode))
		{
			if (childNode instanceof ParameterNode)
			{
				//noinspection unchecked
				getUserObject().moveUp(((ParameterNode)childNode).getUserObject());
			}
			return true;
		}
		return false;
	}

	public boolean moveDown(GenericTreeNode childNode)
	{
		if (super.moveDown(childNode))
		{
			if (childNode instanceof ParameterNode)
			{
				//noinspection unchecked
				getUserObject().moveDown(((ParameterNode)childNode).getUserObject());
			}
			return true;
		}
		return false;
	}
}
