// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.util.Vector;

public class CollectionChangeSupport
{
	private transient Vector<CollectionChangeListener> listeners;

	private Object source;

	public CollectionChangeSupport(Object sourceBean)
	{
		if (sourceBean==null) throw new NullPointerException();
		source=sourceBean;
	}

	public synchronized void addListener(CollectionChangeListener listener)
	{
		if (listener==null) throw new IllegalArgumentException("listener");
		if (listeners==null) listeners=new Vector<CollectionChangeListener>();
		listeners.addElement(listener);
	}

	public synchronized void removeListener(CollectionChangeListener listener)
	{
		if (listeners==null) return;
		listeners.removeElement(listener);
	}

	public synchronized CollectionChangeListener[] getCollectionChangeListeners()
	{
		if (listeners==null) return new CollectionChangeListener[0];
		return listeners.toArray(new CollectionChangeListener[0]);
	}

	public void fireElementAdded(String propertyName, Object newElement)
	{
		CollectionChangeListener[] listeners=getCollectionChangeListeners();
		if (listeners==null || listeners.length==0) return;
		CollectionChangeEvent event=new CollectionChangeEvent(source, propertyName, newElement, CollectionChangeEvent.ADDED);
		for (CollectionChangeListener listener : listeners)
		{
			if (listener!=null) listener.collectionChanged(event);
		}
	}

	public void fireElementRemoved(String propertyName, Object oldElement)
	{
		CollectionChangeListener[] listeners=getCollectionChangeListeners();
		if (listeners==null || listeners.length==0) return;
		CollectionChangeEvent event=new CollectionChangeEvent(source, propertyName, oldElement, CollectionChangeEvent.REMOVED);
		for (int i=0; i<listeners.length; i++)
		{
			CollectionChangeListener listener=listeners[i];
			if (listener!=null) listener.collectionChanged(event);
		}
	}

	public void fireElementChanged(String propertyName, Object oldElement)
	{
		CollectionChangeListener[] listeners=getCollectionChangeListeners();
		if (listeners==null || listeners.length==0) return;
		CollectionChangeEvent event=new CollectionChangeEvent(source, propertyName, oldElement, CollectionChangeEvent.CHANGED);
		for (int i=0; i<listeners.length; i++)
		{
			CollectionChangeListener listener=listeners[i];
			if (listener!=null) listener.collectionChanged(event);
		}
	}
}
