// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.io.InputStream;
import java.io.OutputStream;
import java.io.IOException;
import java.io.Closeable;

/**
 * I/O utility methods
 *
 * @author Stefan Stiller
 */
public class IOUtils
{
	private IOUtils()
	{
	}

	/**
	 * Reads from an input stream until the end and copies everything to the output stream.
	 */
	public static void copyStream(InputStream inputStream, OutputStream outputStream) throws IOException
	{
		byte[] buffer=new byte[1024];
		int length;
		try
		{
			while ((length=inputStream.read(buffer))>=0)
			{
				outputStream.write(buffer, 0, length);
			}
		}
		finally
		{
			tryClose(inputStream);
			tryClose(outputStream);
		}
	}

	/**
	 * Tries to close a stream.
	 */
	public static void tryClose(Closeable closeable)
	{
		try
		{
			closeable.close();
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
}
