// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.beans.PropertyChangeSupport;
import java.beans.PropertyChangeListener;

import org.apache.commons.beanutils.BasicDynaBean;
import org.apache.commons.beanutils.DynaClass;

/**
 * A {@link org.apache.commons.beanutils.DynaBean} with property change support.
 *
 * @author Stefan Stiller
 */
public class ObservableDynaBean extends BasicDynaBean implements ObservableBean
{
	private PropertyChangeSupport changeSupport=new PropertyChangeSupport(this);

	public ObservableDynaBean(DynaClass dynaClass)
	{
		super(dynaClass);
	}

	public void set(String name, Object value)
	{
		Object oldValue=get(name);
		super.set(name, value);
		firePropertyChange(name, oldValue, value);
	}

	public void addPropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(listener);
	}

	public void addPropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.addPropertyChangeListener(propertyName, listener);
	}

	public void removePropertyChangeListener(String propertyName, PropertyChangeListener listener)
	{
		changeSupport.removePropertyChangeListener(propertyName, listener);
	}

	public void firePropertyChange(String propertyName, Object oldValue, Object newValue)
	{
		changeSupport.firePropertyChange(propertyName, oldValue, newValue);
	}

}
