// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils;

import java.awt.Color;
import java.io.File;
import java.io.UnsupportedEncodingException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.net.URL;
import java.net.URLDecoder;
import java.util.SortedSet;
import java.util.Collection;
import java.util.TreeSet;

import com.kiwisoft.utils.format.FormatStringComparator;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.13 $, $Date: 2006/03/24 18:50:51 $
 */
public class Utils
{
	private Utils()
	{
	}

	public static boolean equals(Object o1, Object o2)
	{
		if (o1==o2) return true;
		else if (o1==null) return false; // o2 is not null because of the first test
		else return o1.equals(o2);
	}

	public static String getShortClassName(Class aClass)
	{
		String name=aClass.getName();
		return name.substring(name.lastIndexOf(".")+1);
	}

	public static Class convertPrimitiveClass(Class aClass)
	{
		if (aClass!=null && aClass.isPrimitive())
		{
			if (Boolean.TYPE.equals(aClass)) return Boolean.class;
			if (Byte.TYPE.equals(aClass)) return Byte.class;
			if (Short.TYPE.equals(aClass)) return Short.class;
			if (Integer.TYPE.equals(aClass)) return Integer.class;
			if (Long.TYPE.equals(aClass)) return Long.class;
			if (Float.TYPE.equals(aClass)) return Float.class;
			if (Double.TYPE.equals(aClass)) return Double.class;
			if (Character.TYPE.equals(aClass)) return Character.class;
		}
		return aClass;
	}

	public static Color darker(Color color, double factor)
	{
		return new Color(Math.max((int)(color.getRed()*factor), 0),
						 Math.max((int)(color.getGreen()*factor), 0),
						 Math.max((int)(color.getBlue()*factor), 0));
	}

	public static <T> SortedSet<T> sortedSet(Collection<T> collection)
	{
		TreeSet<T> set=new TreeSet<T>(new FormatStringComparator());
		set.addAll(collection);
		return set;
	}

	public static boolean isJavaIdentifier(String string)
	{
		return string.matches("\\p{javaJavaIdentifierStart}\\p{javaJavaIdentifierPart}*");
	}

	public static String toString(Throwable exception)
	{
		StringWriter stringWriter=new StringWriter();
		exception.printStackTrace(new PrintWriter(stringWriter));
		return stringWriter.toString();
	}
}
