// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.format;

import java.text.NumberFormat;
import java.text.ParseException;
import java.util.Locale;
import java.math.BigDecimal;

import com.kiwisoft.utils.StringUtils;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.3 $, $Date: 2006/03/24 18:33:38 $
 */
public class JavaNumberFormat extends DefaultNumberFormat
{
	private NumberFormat numberFormat;

	public JavaNumberFormat(String name)
	{
		super(name);
		numberFormat=NumberFormat.getInstance(Locale.US);
		numberFormat.setMinimumFractionDigits(0);
		numberFormat.setMaximumFractionDigits(8);
		numberFormat.setMinimumIntegerDigits(1);
		numberFormat.setMaximumIntegerDigits(8);
		numberFormat.setGroupingUsed(false);
		numberFormat.setParseIntegerOnly(false);
	}

	public String format(Object value)
	{
		if (value instanceof Number) return numberFormat.format(value);
		return super.format(value);
	}

	public Object parse(String value, Class targetClass)
	{
		if (StringUtils.isEmpty(value)) return null;
		value=value.trim();
		try
		{
			Number number=numberFormat.parse(value);
			if (number==null || targetClass.isInstance(number)) return number;
			if (targetClass==Double.class) return new Double(number.doubleValue());
			if (targetClass==Float.class) return new Float(number.floatValue());
			if (targetClass==BigDecimal.class) return new BigDecimal(number.doubleValue());
			return number;
		}
		catch (ParseException e)
		{
			throw new RuntimeException(e);

		}
	}
}
