// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui;

import java.awt.*;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeNode;
import javax.swing.*;
import java.util.List;
import java.util.LinkedList;
import java.lang.reflect.Proxy;

import com.kiwisoft.utils.Utils;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.13 $, $Date: 2006/03/24 18:50:51 $
 */
public class GuiUtils
{
	private GuiUtils()
	{
	}

	public static void arrangeWindow(Window parent, Window child)
	{
		Point posParent;
		if (parent!=null)
			posParent=parent.getLocation();
		else
			posParent=new Point();
		Dimension sizeParent;
		if (parent!=null)
			sizeParent=parent.getSize();
		else
			sizeParent=Toolkit.getDefaultToolkit().getScreenSize();
		Dimension sizeChild=child.getSize();
		Point posChild=new Point();
		posChild.x=Math.max(0, posParent.x+(sizeParent.width-sizeChild.width)/2);
		posChild.y=Math.max(0, posParent.y+(sizeParent.height-sizeChild.height)/2);
		child.setLocation(posChild);
	}

	public static JButton createButton(Action action)
	{
		JButton button=new JButton(action);
		button.setMargin(new Insets(0, 0, 0, 0));
		button.setText(null);
		button.setFocusable(false);
		KeyStroke keyStroke=(KeyStroke)action.getValue(Action.ACCELERATOR_KEY);
		if (keyStroke!=null)
		{
			InputMap inputMap=button.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
			inputMap.put(keyStroke, Action.ACCELERATOR_KEY);
			button.getActionMap().put(Action.ACCELERATOR_KEY, action);
		}
		return button;
	}

	public static JToggleButton createToggleButton(Action action)
	{
		JToggleButton button=new JToggleButton(action);
		button.setMargin(new Insets(1, 1, 1, 1));
		button.setText(null);
		button.setBorderPainted(false);
		KeyStroke keyStroke=(KeyStroke)action.getValue(Action.ACCELERATOR_KEY);
		if (keyStroke!=null)
		{
			InputMap inputMap=button.getInputMap(JComponent.WHEN_IN_FOCUSED_WINDOW);
			inputMap.put(keyStroke, Action.ACCELERATOR_KEY);
			button.getActionMap().put(Action.ACCELERATOR_KEY, action);
		}
		return button;
	}

	public static void handleThrowable(Component component, Throwable throwable)
	{
		throwable.printStackTrace();
		JOptionPane.showMessageDialog(component, Utils.getShortClassName(throwable.getClass()), throwable.getMessage(),
									  JOptionPane.ERROR_MESSAGE);
	}

	public static TreePath getPath(TreeNode node)
	{
		List<TreeNode> path=new LinkedList<TreeNode>();
		while (node!=null)
		{
			path.add(0, node);
			node=node.getParent();
		}
		return new TreePath(path.toArray());
	}
}
