// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui;

import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.kiwisoft.utils.xml.XMLAdapter;
import com.kiwisoft.utils.xml.XMLContext;
import com.kiwisoft.utils.xml.XMLHandler;
import com.kiwisoft.utils.xml.XMLObject;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.4 $, $Date: 2006/03/24 18:52:26 $
 */
public class Icons
{
	public static final Icon ICON_16X16=new ImageIcon(new BufferedImage(16, 16, BufferedImage.TYPE_4BYTE_ABGR));

	private static Hashtable<String, Icon> icons;
	private static Map<String, IconResource> iconMap;

	private static Map<String, IconResource> getIconMap()
	{
		if (iconMap==null)
		{
			iconMap=new HashMap<String, IconResource>();
			load("/icons/Icons.xml");
		}
		return iconMap;
	}

	private static void load(String resource)
	{
		XMLHandler xmlHandler=new XMLHandler();
		xmlHandler.addTagMapping("icons", new IconsAdapter());
		xmlHandler.addTagMapping("include", IncludeAdapter.class);
		xmlHandler.addTagMapping("icon", IconAdapter.class);
		xmlHandler.addTagMapping("resource", Resource.class);
		xmlHandler.addTagMapping("merge", Merged.class);
		try
		{
			xmlHandler.loadStream(Icons.class.getResourceAsStream(resource));
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public static Icon getIcon(String key)
	{
		if (icons==null) icons=new Hashtable<String, Icon>();
		Icon icon=icons.get(key);
		if (icon==null)
		{
			IconResource resource=getIconMap().get(key);
			icon=resource.getIcon();
			icons.put(key, icon);
		}
		return icon;
	}

	public static ImageIcon loadIcon(String resource)
	{
		return new ImageIcon(Icons.class.getResource(resource));
	}

	private Icons()
	{
	}

	public static class IconsAdapter extends XMLAdapter
	{
		public void addXMLElement(XMLContext context, XMLObject element)
		{
			if (element instanceof IncludeAdapter)
			{
				IncludeAdapter includeAdapter=(IncludeAdapter)element;
				load(includeAdapter.getResource());
			}
			else if (element instanceof IconAdapter)
			{
				IconAdapter iconAdapter=(IconAdapter)element;
				iconMap.put(iconAdapter.getKey(), iconAdapter.getIconResource());
			}
		}
	}

	public static class IncludeAdapter extends XMLAdapter
	{
		private String resource;

		public IncludeAdapter(XMLContext context, String name)
		{
			super(context, name);
		}

		public void setXMLAttribute(XMLContext context, String name, String value)
		{
			if ("resource".equalsIgnoreCase(name)) this.resource=value;
		}

		public String getResource()
		{
			return resource;
		}
	}

	public static class IconAdapter extends XMLAdapter
	{
		private String key;
		private IconResource iconResource;

		public IconAdapter(XMLContext context, String name)
		{
			super(context, name);
		}

		public void setXMLAttribute(XMLContext context, String name, String value)
		{
			if ("key".equalsIgnoreCase(name)) key=value;
		}

		public void addXMLElement(XMLContext context, XMLObject element)
		{
			if (element instanceof IconResource) this.iconResource=(IconResource)element;
		}

		public String getKey()
		{
			return key;
		}

		public IconResource getIconResource()
		{
			return iconResource;
		}
	}

	public interface IconResource
	{
		Icon getIcon();
	}

	public static class Resource extends XMLAdapter implements IconResource
	{
		private String url;

		public Resource(XMLContext context, String name)
		{
			super(context, name);
		}

		public void setXMLAttribute(XMLContext context, String name, String value)
		{
			if ("url".equalsIgnoreCase(name)) this.url=value;
		}

		public Icon getIcon()
		{
			return loadIcon(url);
		}
	}

	public static class Merged extends XMLAdapter implements IconResource
	{
		private String icon1;
		private String icon2;
		private int offsetX;
		private int offsetY;

		public Merged(XMLContext context, String name)
		{
			super(context, name);
		}

		public void setXMLAttribute(XMLContext context, String name, String value)
		{
			if ("icon1".equalsIgnoreCase(name)) icon1=value;
			else if ("icon2".equalsIgnoreCase(name)) icon2=value;
			else if ("offsetX".equalsIgnoreCase(name)) offsetX=Integer.parseInt(value);
			else if ("offsetY".equalsIgnoreCase(name)) offsetY=Integer.parseInt(value);
		}

		public Icon getIcon()
		{
			return new MergedIcon(Icons.getIcon(icon1), Icons.getIcon(icon2), offsetX, offsetY);
		}
	}
}
