// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.adapters;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeEvent;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JCheckBox;

import org.apache.commons.beanutils.PropertyUtils;

import com.kiwisoft.utils.ObservableBean;

/**
 * An adapter used for binding a checkbox to a boolean property. 
 *
 * @author Stefan Stiller
 */
public class CheckBoxAdapter implements PropertyChangeListener, ActionListener, ComponentAdapter
{
	private JCheckBox checkBox;
	private ObservableBean object;
	private String property;

	public CheckBoxAdapter(JCheckBox checkBox)
	{
		this.checkBox=checkBox;
	}

	public void bind(ObservableBean object, String property)
	{
		this.object=object;
		this.property=property;
		boolean writeable=false;
		checkBox.setSelected(false);
		if (object!=null)
		{
			boolean readable=PropertyUtils.isReadable(object, property);
			try
			{
				Class propertyType=PropertyUtils.getPropertyType(object, property);
				writeable=readable && PropertyUtils.isWriteable(object, property)
						  && (propertyType==Boolean.class || propertyType==Boolean.TYPE);
			}
			catch (Exception e)
			{
				e.printStackTrace();
			}
			if (readable)
			{
				updateComponent();
				object.addPropertyChangeListener(property, this);
			}
			if (writeable)
			{
				checkBox.addActionListener(this);
			}
		}
		checkBox.setEnabled(writeable);
	}

	private void updateComponent()
	{
		try
		{
			Boolean value=(Boolean)PropertyUtils.getProperty(object, property);
			checkBox.setSelected(value!=null && value.booleanValue());
		}
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}

	public void unbind()
	{
		if (object!=null) object.removePropertyChangeListener(property, this);
		checkBox.removeActionListener(this);
	}

	public void propertyChange(PropertyChangeEvent evt)
	{
		updateComponent();
	}

	public void actionPerformed(ActionEvent e)
	{
		if (object!=null && checkBox.isEnabled())
		{
			try
			{
				PropertyUtils.setProperty(object, property, checkBox.isSelected());
			}
			catch (Exception e1)
			{
				e1.printStackTrace();
			}
		}
	}

	public boolean stopEditing()
	{
		return true;
	}
}
