// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.list;

import java.awt.Component;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JList;
import javax.swing.UIManager;

import com.kiwisoft.utils.format.ObjectFormat;
import com.kiwisoft.utils.format.FormatManager;

/**
 * A list cell renderer based on {@link ObjectFormat}. 
 *
 * @author Stefan Stiller
 */
public class FormatListCellRenderer extends DefaultListCellRenderer
{
	public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus)
	{
		setComponentOrientation(list.getComponentOrientation());
		if (isSelected)
		{
			setBackground(list.getSelectionBackground());
			setForeground(list.getSelectionForeground());
		}
		else
		{
			setBackground(list.getBackground());
			setForeground(list.getForeground());
		}

		if (value!=null)
		{
			ObjectFormat format=FormatManager.getInstance().getFormat(value.getClass());
			setIcon(format.getIcon(value));
			Object text=format.format(value);
			setText(text!=null ? text.toString() : "");
		}
		else
		{
			setIcon(null);
			setText("");
		}
		setEnabled(list.isEnabled());
		setFont(list.getFont());
		setBorder((cellHasFocus) ? UIManager.getBorder("List.focusCellHighlightBorder") : noFocusBorder);
		return this;
	}
}
