// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.lookup;

import java.awt.BorderLayout;
import java.awt.Point;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.util.Collection;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import javax.swing.AbstractAction;
import javax.swing.DefaultListModel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JWindow;
import javax.swing.KeyStroke;

import com.kiwisoft.utils.gui.list.FormatListCellRenderer;
import com.kiwisoft.utils.format.FormatStringComparator;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.2 $, $Date: 2006/03/24 18:04:41 $
 */
public abstract class ListLookup<T> implements Lookup<T>
{
	protected ListLookup()
	{
	}

	public void open(LookupField<T> field, Collection<T> values)
	{
		LookupWindow window = new LookupWindow(field, values);
		Point point=field.getLocationOnScreen();
		window.setLocation(point.x, point.y+field.getSize().height);
		window.setSize(300, 200);
		window.setVisible(true);
		window.toFront();
		window.setFocusableWindowState(true);
		window.requestFocus();
		window.list.requestFocus();
		window.list.setSelectedValue(field.getValue(), true);
		window.addWindowFocusListener(window);
	}

	public class CancelAction extends AbstractAction
	{
		private LookupField field;

		public CancelAction(LookupField field)
		{
			this.field=field;
		}

		public void actionPerformed(ActionEvent e)
		{
			field.requestFocus();
		}
	}

	public class SelectionListener extends AbstractAction implements MouseListener
	{
		private JList list;
		private LookupField<T> field;

		public SelectionListener(JList list, LookupField<T> field)
		{
			this.list=list;
			this.field=field;
		}

		public void actionPerformed(ActionEvent e)
		{
			//noinspection unchecked
			T value=(T)list.getSelectedValue();
			if (value!=null)
			{
				field.setValue(value);
				field.requestFocus();
			}
		}

		public void mouseClicked(MouseEvent e)
		{
			if (e.getClickCount()>1 && e.getButton()==MouseEvent.BUTTON1)
			{
				//noinspection unchecked
				T value=(T)list.getSelectedValue();
				if (value!=null)
				{
					field.setValue(value);
					field.requestFocus();
				}
				e.consume();
			}
		}

		public void mousePressed(MouseEvent e)
		{
		}

		public void mouseReleased(MouseEvent e)
		{
		}

		public void mouseEntered(MouseEvent e)
		{
		}

		public void mouseExited(MouseEvent e)
		{
		}
	}

	public class LookupWindow extends JWindow implements WindowFocusListener
	{
		private JList list;
		private LookupField<T> field;

		public LookupWindow(LookupField<T> field, Collection<T> values)
		{
			super((Window)field.getTopLevelAncestor());
			this.field=field;
			setContentPane(createContentPanel(values));
		}

		private JPanel createContentPanel(Collection<T> values)
		{
			JPanel panel=new JPanel(new BorderLayout());
			List<T> valueList=Collections.emptyList();
			if (values!=null)
			{
				valueList = new LinkedList<T>(values);
				Collections.sort(valueList, new FormatStringComparator());
			}
			DefaultListModel model=new DefaultListModel();
			for (T element : valueList) model.addElement(element);
			list=new JList(model);
			list.setCellRenderer(new FormatListCellRenderer());
			SelectionListener selectionListener=new SelectionListener(list, field);
			list.getInputMap().put(KeyStroke.getKeyStroke(KeyEvent.VK_ENTER, 0), "select");
			list.getActionMap().put("select", selectionListener);
			list.getInputMap(JList.WHEN_IN_FOCUSED_WINDOW).put(KeyStroke.getKeyStroke(KeyEvent.VK_ESCAPE, 0), "cancel");
			list.getActionMap().put("cancel", new CancelAction(field));
			list.addMouseListener(selectionListener);
			panel.add(new JScrollPane(list), BorderLayout.CENTER);
			return panel;
		}

		public void windowGainedFocus(WindowEvent e)
		{
		}

		public void windowLostFocus(WindowEvent e)
		{
			dispose();
		}

		public void dispose()
		{
			field=null;
			list=null;
			super.dispose();
		}
	}
}
