// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.util.EventListener;
import javax.swing.event.EventListenerList;

import com.kiwisoft.utils.gui.ObjectStyle;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.5 $, $Date: 2006/03/24 18:58:22 $
 */
public abstract class SortableTableRow<T>
{
	private T userObject;
	protected final EventListenerList listeners=new EventListenerList();

	protected SortableTableRow(T userObject)
	{
		this.userObject=userObject;
	}

	public Class getCellClass(int col)
	{
		Object value=getDisplayValue(col);
		if (value!=null) return value.getClass();
		return null;
	}

	public boolean isEditable(int column)
	{
		return false;
	}

	public Comparable getSortValue(int col)
	{
	    Object value=getDisplayValue(col);
	    if (value instanceof Comparable)
	        return (Comparable) value;
	    else if (value==null)
	        return "";
	    else
	        return String.valueOf(value);
	}

    public abstract Object getDisplayValue(int column);

	public int setValue(Object value, int col)
	{
		return SortableTableModel.NO_UPDATE;
	}

	public T getUserObject()
	{
		return userObject;
	}

	public void setUserObject(T userObject)
	{
		this.userObject=userObject;
	}

	public ObjectStyle getCellStyle(int col)
	{
		return null;
	}

	public String getCellFormat(int col)
	{
		return null;
	}

	public Object getToolTipText(int col)
	{
		return getDisplayValue(col);
	}

	public void installListener()
	{
	}

	public void removeListener()
	{
	}

	public void addRowListener(TableRowListener listener)
	{
		listeners.add(TableRowListener.class, listener);
	}

	public void removeRowListener(TableRowListener listener)
	{
		listeners.remove(TableRowListener.class, listener);
	}

	protected void fireRowUpdated()
	{
		EventListener[] rowListeners=listeners.getListeners(TableRowListener.class);
		for (int i=0; i<rowListeners.length; i++)
		{
			TableRowListener listener=(TableRowListener)rowListeners[i];
			listener.rowUpdated(this);
		}
	}

	protected void fireRowDeleted()
	{
		EventListener[] rowListeners=listeners.getListeners(TableRowListener.class);
		for (int i=0; i<rowListeners.length; i++)
		{
			TableRowListener listener=(TableRowListener)rowListeners[i];
			listener.rowDeleted(this);
		}
	}

	protected int getSortPriority()
	{
		return 0;
	}

	public ObjectEditor getObjectEditor(int col)
	{
		return null;
	}
}
