// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.util.*;
import javax.swing.JComponent;
import javax.swing.table.TableCellRenderer;

import com.kiwisoft.utils.*;
import com.kiwisoft.utils.format.*;
import com.kiwisoft.utils.gui.ComponentDecorator;
import com.kiwisoft.utils.gui.ObjectStyle;

/**
 * @author Stefan Stiller
 * @version $Revision: 1.5 $, $Date: 2006/03/24 18:58:21 $
 */
public class TableRendererFactory extends ClassFeatureManager<ObjectRenderer>
{
	private static TableRendererFactory instance;

	public synchronized static TableRendererFactory getInstance()
	{
		if (instance==null) instance=new TableRendererFactory();
		return instance;
	}

	private DoubleKeyMap<ObjectRenderer, ObjectStyle, StyleRenderer> styleRenderers=new DoubleKeyMap<ObjectRenderer, ObjectStyle, StyleRenderer>();

	private TableRendererFactory()
	{
		setFeature(Object.class, new DefaultObjectRenderer(new DefaultObjectFormat(DEFAULT)));
		setFeature(Number.class, new DefaultObjectRenderer(new DefaultNumberFormat(DEFAULT)));
		setFeature(Date.class, new DefaultObjectRenderer(new DefaultDateFormat(DEFAULT)));
		setFeature(String.class, new DefaultObjectRenderer(new DefaultStringFormat(DEFAULT)));
		setFeature(Boolean.class, new BooleanObjectRenderer());
		TristateObjectRenderer tristateObjectRenderer=new TristateObjectRenderer();
		setFeature(Boolean.class, "Tristate", tristateObjectRenderer);
		setFeature(Tristate.class, tristateObjectRenderer);
	}

	public ObjectRenderer getRenderer(Class aClass, String variant)
	{
		return getFeature(aClass, variant);
	}

	protected ObjectRenderer getInheritedDefaultFeature(Class aClass)
	{
		ObjectFormat format=FormatManager.getInstance().getRegisteredFormat(aClass);
		if (format!=null) return createFormatRenderer(format);
		return super.getInheritedDefaultFeature(aClass);
	}

	protected ObjectRenderer getInheritedVariantFeature(Class aClass, String variant)
	{
		ObjectFormat format=FormatManager.getInstance().getRegisteredFormat(aClass, variant);
		if (format!=null) return createFormatRenderer(format);
		return super.getInheritedVariantFeature(aClass, variant);
	}

	private ObjectRenderer createFormatRenderer(ObjectFormat format)
	{
		if (format instanceof BooleanFormat)
			return new BooleanObjectRenderer((BooleanFormat)format);
		else
			return new DefaultObjectRenderer((TextFormat)format);
	}

	public TableCellRenderer getRenderer(Class aClass, String variant, ObjectStyle style)
	{
		aClass=Utils.convertPrimitiveClass(aClass);
		ObjectRenderer renderer=getRenderer(aClass, variant);
		if (style==null) return renderer;
		StyleRenderer styleRenderer=styleRenderers.get(renderer, style);
		if (styleRenderer==null)
		{
			styleRenderer=new StyleRenderer(renderer, style);
			styleRenderers.put(renderer, style, styleRenderer);
		}
		return styleRenderer;
	}

	public void setRenderer(Class aClass, ObjectFormat format)
	{
		ObjectRenderer renderer;
		renderer=createFormatRenderer(format);
		setFeature(aClass, format.getName(), renderer);
	}

	public void setRenderer(Class aClass, String variant, ObjectRenderer renderer)
	{
		setFeature(aClass, variant, renderer);
	}

	public static void initializeRowColor(JComponent component, ObjectRenderer renderer, int row)
	{
		Color color=(Color)component.getClientProperty(SortableTable.ALTERNATE_ROW_BACKGROUND);
		if (color!=null && row%2==0) renderer.setRowBackground(null);
		else renderer.setRowBackground(color);
		color=(Color)component.getClientProperty(SortableTable.ALTERNATE_ROW_FOREGROUND);
		if (color!=null && row%2==0) renderer.setRowForeground(null);
		else renderer.setRowForeground(color);
	}

	public static void decorateComponent(Graphics g, Component component, List<ComponentDecorator> decorators)
	{
		if (decorators!=null)
		{
			for (ComponentDecorator decorator : decorators) decorator.decorateComponent(g, component);
		}
	}

	public static void prepareComponent(Graphics g, Component component, List<ComponentDecorator> decorators)
	{
		if (decorators!=null)
		{
			for (ComponentDecorator decorator : decorators) decorator.prepareComponent(g, component);
		}
	}

	public static void paintBackground(Graphics g, JComponent component, Color cellBackground, Color rowBackground)
	{
		component.setOpaque(false);
		Dimension size=component.getSize();
		if (cellBackground!=null && rowBackground!=null)
		{
			for (int i=0; i<size.width+size.height; i++)
			{
				if (i%4<2) g.setColor(cellBackground);
				else g.setColor(rowBackground);
				g.drawLine(i, 0, i-size.height, size.height);
			}
		}
		else
		{
			g.setColor(component.getBackground());
			g.fillRect(0, 0, size.width, size.height);
		}
	}
}
