// Copyright (c) 2002-2006, Stefan Stiller.
package com.kiwisoft.utils.gui.tree;

import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import javax.swing.Action;
import javax.swing.JPopupMenu;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import org.jetbrains.annotations.NotNull;

import com.kiwisoft.utils.Disposable;

/**
 * JTree implementation which uses Java 1.5 generics, some better tree node
 * handling, and context menu.
 *
 * @author Stefan Stiller
 */
public class GenericTree extends JTree implements MouseListener, Disposable
{
	/**
	 * Creates a new tree with the given node as its root.
	 *
	 * @param rootNode The root node.
	 */
	public GenericTree(@NotNull GenericTreeNode rootNode)
	{
		DefaultTreeModel treeModel=new DefaultTreeModel(rootNode);
		rootNode.setTreeModel(treeModel);
		setModel(treeModel);
		setCellRenderer(new GenericTreeNodeRenderer());
		addMouseListener(this);
	}

	public GenericTreeNode<?> getSelectedNode()
	{
		TreePath treePath=getLeadSelectionPath();
		if (treePath==null) return null;
		return (GenericTreeNode)treePath.getLastPathComponent();
	}

	public void mouseClicked(MouseEvent e)
	{
		if (e.getButton()==MouseEvent.BUTTON3 && e.getClickCount()==1)
		{
			TreePath path=getPathForLocation(e.getX(), e.getY());
			if (path!=null)
			{
				GenericTreeNode treeNode=(GenericTreeNode)path.getLastPathComponent();
				Action[] actions=treeNode.getActions(this);
				if (actions!=null && actions.length>0)
				{
					JPopupMenu menu=new JPopupMenu();
					for (int i=0; i<actions.length; i++)
					{
						Action action=actions[i];
						if (action==null) menu.addSeparator();
						else menu.add(action);
					}
					menu.show(this, e.getX(), e.getY());
				}
			}
			e.consume();
		}
	}

	public void mousePressed(MouseEvent e)
	{
	}

	public void mouseReleased(MouseEvent e)
	{
	}

	public void mouseEntered(MouseEvent e)
	{
	}

	public void mouseExited(MouseEvent e)
	{
	}

	public void dispose()
	{
		GenericTreeNode root=(GenericTreeNode)treeModel.getRoot();
		if (root!=null) root.dispose();
	}
}
